unit CPUDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, StdCtrls, ComCtrls, ToolWin, Grids;

type
  TCPUForm = class(TForm)
    ToolBar1: TToolBar;
    Carry: TToolButton;
    Tab: TStringGrid;
    Aux: TToolButton;
    F0: TToolButton;
    RS1: TToolButton;
    RS0: TToolButton;
    Overflow: TToolButton;
    F1: TToolButton;
    Parity: TToolButton;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FlagClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure TabDblClick(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
    Procedure ReadData;
  end;

var
  CPUForm: TCPUForm = nil;

implementation

uses MainUnit, ALU, InputDlg;

{$R *.DFM}

procedure TCPUForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  CPUForm:=nil;
  Action:=caFree;
end;

Procedure TCPUForm.ReadData;
Var I,X : Integer;
begin
  Carry.Down:=(fCarry in MainForm.System8051.ALU.Reg.Flags);
  Aux.Down:=(fAux in MainForm.System8051.ALU.Reg.Flags);
  F0.Down:=(fF0 in MainForm.System8051.ALU.Reg.Flags);
  RS1.Down:=(fRS1 in MainForm.System8051.ALU.Reg.Flags);
  RS0.Down:=(fRS0 in MainForm.System8051.ALU.Reg.Flags);
  Overflow.Down:=(fOverflow in MainForm.System8051.ALU.Reg.Flags);
  F1.Down:=(fF1 in MainForm.System8051.ALU.Reg.Flags);
  Parity.Down:=(fParity in MainForm.System8051.ALU.Reg.Flags);

  I:=MainForm.System8051.ALU.Reg.ACC;
  Tab.Cells[1,1]:='0x'+IntToHex(I,2)+' ('+IntToStr(I)+')';

  I:=MainForm.System8051.ALU.Reg.B;
  Tab.Cells[1,2]:='0x'+IntToHex(I,2)+' ('+IntToStr(I)+')';

  I:=MainForm.System8051.ALU.Reg.PC;
  Tab.Cells[1,3]:='0x'+IntToHex(I,4)+' ('+IntToStr(I)+')';

  I:=MainForm.System8051.ALU.Reg.PSW;
  Tab.Cells[1,4]:='0x'+IntToHex(I,2)+' ('+IntToStr(I)+')';

  I:=MainForm.System8051.ALU.Reg.DPTR;
  Tab.Cells[1,5]:='0x'+IntToHex(I,2)+' ('+IntToStr(I)+')';

  I:=MainForm.System8051.ALU.Reg.SP;
  Tab.Cells[1,6]:='0x'+IntToHex(I,2)+' ('+IntToStr(I)+')';

  For X:=0 to 7 do begin
    I:=MainForm.System8051.ALU.Reg.R[X];
    Tab.Cells[1,7+X]:='0x'+IntToHex(I,2)+' ('+IntToStr(I)+')';
  end;

end;

procedure TCPUForm.FormCreate(Sender: TObject);
Var X : Integer;
begin
  Tab.Cells[0,0]:='Register';
  Tab.Cells[1,0]:='Wert';
  Tab.Cells[0,1]:='Akku';
  Tab.Cells[0,2]:='B';
  Tab.Cells[0,3]:='PC';
  Tab.Cells[0,4]:='PSW (Flags)';
  Tab.Cells[0,5]:='DPTR';
  Tab.Cells[0,6]:='SP';
  for X:=0 to 7 do Tab.Cells[0,7+X]:='R'+IntToStr(X);
  ReadData;
  If MainForm.WindowStyle.ItemIndex>=0 then
    BoundsRect:=MainForm.SetupData.WindowSetup[MainForm.WindowStyle.ItemIndex].CPUWnd;
end;

procedure TCPUForm.FlagClick(Sender: TObject);
Var Flags : TFlagSet;
begin
  Flags:=[];
  If Carry.Down then Flags:=Flags+[fCarry];
  If Aux.Down then Flags:=Flags+[fAux];
  If F0.Down then Flags:=Flags+[fF0];
  If RS1.Down then Flags:=Flags+[fRS1];
  If RS0.Down then Flags:=Flags+[fRS0];
  If Overflow.Down then Flags:=Flags+[fOverflow];
  If F1.Down then Flags:=Flags+[fF1];
  If Parity.Down then Flags:=Flags+[fParity];
  MainForm.System8051.ALU.Reg.Flags:=Flags;
  MainForm.ReadData;
end;

procedure TCPUForm.FormResize(Sender: TObject);
begin
  Tab.ColWidths[0]:=Tab.ClientWidth div 3;
  Tab.ColWidths[1]:=2*Tab.ClientWidth div 3;
end;

procedure TCPUForm.TabDblClick(Sender: TObject);
Var I : Integer;
begin
  If Tab.Row<1 then exit;
  Case Tab.Row of
    1 : begin
          I:=MainForm.System8051.ALU.Reg.ACC;
          If InputNumber(I,1) then MainForm.System8051.ALU.Reg.ACC:=I;
        end;
    2 : begin
          I:=MainForm.System8051.ALU.Reg.B;
          If InputNumber(I,1) then MainForm.System8051.ALU.Reg.B:=I;
        end;
    3 : begin
          I:=MainForm.System8051.ALU.Reg.PC;
          If InputNumber(I,2) then MainForm.System8051.ALU.Reg.PC:=I;
        end;
    4 : begin
          I:=MainForm.System8051.ALU.Reg.PSW;
          If InputNumber(I,2) then MainForm.System8051.ALU.Reg.PSW:=I;
        end;
    5 : begin
          I:=MainForm.System8051.ALU.Reg.DPTR;
          If InputNumber(I,2) then MainForm.System8051.ALU.Reg.DPTR:=I;
        end;
    6 : begin
          I:=MainForm.System8051.ALU.Reg.SP;
          If InputNumber(I,1) then MainForm.System8051.ALU.Reg.SP:=I;
         end;
    7..14 : begin
              I:=MainForm.System8051.ALU.Reg.R[Tab.Row-7];
              If InputNumber(I,2) then MainForm.System8051.ALU.Reg.R[Tab.Row-7]:=I;
            end;
  end;
  MainForm.ReadData;
end;

end.
