unit Tools;
interface

uses Classes, SysUtils;

Type ESystemError=class(Exception);

Type ECompilerError=class(Exception)
       private
         FLine : Integer;
       public
         Constructor Create(Msg : String; ALine : Integer);
       published
         property Line : Integer read FLine;
     end;

Function Left(SubStr,S : String) : Boolean;

Function ExtUpperCase(const S : String) : String;
Function ExtLowerCase(const S : String) : String;

Function IsByteZahl(S : String; PreChar : Char = #0) : Boolean;
Function IsWordZahl(S : String; PreChar : Char = #0) : Boolean;
Function IsShortZahl(S : String; PreChar : Char = #0) : Boolean;

Function BinToText(St : TStream) : String;
Function BinToSRec2(const St : TStrings; Start : Integer) : TStrings;

Function BinToInt(S : String; Var I : Integer) : Boolean;
Function IntToBin(I : Integer) : String;

Function ZweiKomplement(I,Bytes : Integer) : Integer;
Function ZweiKomplementBack(I,Bytes : Integer) : Integer;

Function IsRx(const S : String) : Boolean;

Function IsBitAddr(S : String) : Boolean;
Function GetBitAddr(S : String) : Byte;
Function CodeToBit(B : Byte) : Byte;

Procedure StrictAllTimeTab(B : Boolean);
Function IsStrictAllTimeTab : Boolean;
Var AllTimeTab, AllTimeBitTab : TStrings;

implementation

uses PrgConst, Dialogs;

Constructor ECompilerError.Create(Msg : String; ALine : Integer);
begin
  inherited Create(Msg); FLine:=ALine;
end;

Function Left(SubStr,S : String) : Boolean;
begin
  result:=(Copy(S,1,length(SubStr))=SubStr);
end;

Function ExtUpperCase(const S : String) : String;
Var X : Integer;
begin
  result:=S;
  For X:=1 to length(result) do Case result[X] of
    'a'..'z' : dec(result[x],32);
    '' : result[x]:='';
    '' : result[x]:='';
    '' : result[x]:='';
  end;
end;

Function ExtLowerCase(const S : String) : String;
Var X : Integer;
begin
  result:=S;
  For X:=1 to length(result) do Case result[X] of
    'A'..'Z' : inc(result[x],32);
    '' : result[x]:='';
    '' : result[x]:='';
    '' : result[x]:='';
  end;
end;

Function IsByteZahl(S : String; PreChar : Char = #0) : Boolean;
Var I : Integer;
begin
  result:=False;
  S:=Trim(S);
  If length(S)=0 then exit;
  If PreChar<>#0 then begin
    If S[1]<>PreChar then exit;
    S:=Trim(Copy(S,2,MaxInt));
    If length(S)=0 then exit;
  end;
  try I:=StrToInt(S); except exit; end;
  If (I<0) or (I>255) then exit;
  result:=True;
end;

Function IsWordZahl(S : String; PreChar : Char = #0) : Boolean;
Var I : Integer;
begin
  result:=False;
  S:=Trim(S);
  If length(S)=0 then exit;
  If PreChar<>#0 then begin
    If S[1]<>PreChar then exit;
    S:=Trim(Copy(S,2,MaxInt));
    If length(S)=0 then exit;
  end;
  try I:=StrToInt(S); except exit; end;
  If (I<0) or (I>65535) then exit;
  result:=True;
end;

Function IsShortZahl(S : String; PreChar : Char = #0) : Boolean;
Var I : Integer;
begin
  result:=False;
  S:=Trim(S);
  If length(S)=0 then exit;
  If PreChar<>#0 then begin
    If S[1]<>PreChar then exit;
    S:=Trim(Copy(S,2,MaxInt));
    If length(S)=0 then exit;
  end;
  try I:=StrToInt(S); except exit; end;
  If (I<-128) or (I>127) then exit;
  result:=True;
end;


Function BinToText(St : TStream) : String;
Var B : Byte;
begin
  result:='';
  while St.Position<St.Size do begin
    St.Read(B,1);
    result:=result+IntToHex(B,2);
  end;
end;

Function AddCheckSum(S : String) : String;
Var X,I : Integer;
begin
  {Alle Bytes aufaddieren; CheckSum=255-(I mod 256)}
  I:=0; For X:=0 to length(S) div 2-1 do inc(I,StrToInt('0x'+Copy(S,X*2+1,2)));
  result:=S+IntToHex(255-(I mod 256),2);
end;

Function BuildLine(S : String; Var Start : Integer) : String;
begin
  result:='S1'+AddCheckSum(IntToHex(length(S) div 2+3,2)+IntToHex(Start,4)+S);
end;

Function BinToSRec2(const St : TStrings; Start : Integer) : TStrings;
Var I,X : Integer;
begin
  result:=TStringlist.Create; result.Add('lo'); result.Add('S0030000FC'); X:=Start;
  For I:=0 to St.Count-1 do result.Add(BuildLine(St[I],X));
  result.Add('S9'+AddCheckSum('03'+IntToHex(Start,4)));
end;

Function BinToInt(S : String; Var I : Integer) : Boolean;
begin
  result:=False; I:=0; S:=Trim(S);
  while S<>'' do begin
    I:=I*2;
    Case S[1] of
      '0' : ;
      '1' : inc(I);
      else exit;
    end;
    S:=Copy(S,2,MaxInt);
  end;
  result:=True;
end;

Function IntToBin(I : Integer) : String;
begin
  result:='';
  while I<>0 do begin
    If I mod 2=0 then result:='0'+result else result:='1'+result;
    I:=I div 2;
  end;
end;

Function ZweiKomplement(I,Bytes : Integer) : Integer;
begin
  If I<0 then begin
    I:=abs(I);
    result:=I xor $FFFF;
    inc(result);
    If Bytes=1 then result:=result mod 256 else result:=result mod 65536;
  end else result:=I;  
end;

Function ZweiKomplementBack(I,Bytes : Integer) : Integer;
Var Max : Integer;
begin
  Max:=(1 shl (8*Bytes))-1;
  If I>Max div 2 then begin
    dec(I);
    If Bytes=1 then result:=I xor $FF else result:=I xor $FFFF;
    result:=-(result mod Max);
  end else begin
    result:=I;
  end;
end;

Function IsRx(const S : String) : Boolean;
begin
  result:=(length(S)=2) and (S[1]='r') and (S[2]>='0') and (S[2]<='7');
end;

Function IsBitAddr(S : String) : Boolean;
Var T : String;
    I,J,X : Integer;
    ok : Boolean;
begin
  result:=False;
  If Pos('.',S)=0 then begin
    result:=IsByteZahl(S) or (AllTimeBitTab.IndexOf(ExtLowerCase(S))>=0); exit;
  end;

  T:=ExtLowerCase(Trim(Copy(S,Pos('.',S)+1,MaxInt)));
  S:=ExtLowerCase(Trim(Copy(S,1,Pos('.',S)-1)));

  If not IsByteZahl(T) then exit;
  J:=StrToInt(T); If J>7 then exit;

  If AllTimeTab.IndexOf(S)>=0 then begin
    I:=Integer(AllTimeTab.Objects[AllTimeTab.IndexOf(S)]);
  end else begin
    If not IsByteZahl(S) then exit;
    I:=StrToInt(S);
  end;

  ok:=False;
  For X:=0 to High(BitReg) do If I=BitReg[X] then begin OK:=True; break; end;
  If not ok then exit;

  result:=True;
end;

Function GetBitAddr(S : String) : Byte;
Var T : String;
    I,J,X : Integer;
    ok : Boolean;
begin
  result:=0;
  If Pos('.',S)=0 then begin
    If IsByteZahl(S) then result:=StrToInt(S) else
    result:=Integer(AllTimeBitTab.Objects[AllTimeBitTab.IndexOf(ExtLowerCase(S))]);
    exit;
  end;

  T:=ExtLowerCase(Trim(Copy(S,Pos('.',S)+1,MaxInt)));
  S:=ExtLowerCase(Trim(Copy(S,1,Pos('.',S)-1)));

  If not IsByteZahl(T) then exit;
  J:=StrToInt(T); If J>7 then exit;

  If AllTimeTab.IndexOf(S)>=0 then begin
    I:=Integer(AllTimeTab.Objects[AllTimeTab.IndexOf(S)]);
  end else begin
    If not IsByteZahl(S) then exit;
    I:=StrToInt(S);
  end;

  ok:=False;
  For X:=0 to High(BitReg) do If I=BitReg[X] then begin result:=BitCode[X]; OK:=True; break; end;
  If not ok then exit;
  inc(result,J);
end;

Function CodeToBit(B : Byte) : Byte;
Var X : Integer;
begin
  B:=(B div 8)*8; result:=0;
  For X:=0 to High(BitCode) do If B=BitCode[X] then begin result:=BitReg[X]; exit; end;
end;

Var AllTimeTabAll, AllTimeTabStrict : TStrings;
    AllTimeBitTabAll, AllTimeBitTabStrict : TStrings;

Procedure StrictAllTimeTab(B : Boolean);
begin
  If B then begin
    AllTimeTab:=AllTimeTabStrict;
    AllTimeBitTab:=AllTimeBitTabStrict;
  end else begin
    AllTimeTab:=AllTimeTabAll;
    AllTimeBitTab:=AllTimeBitTabAll;
  end;
end;

Function IsStrictAllTimeTab : Boolean;
begin
  result:=(AllTimeTab=AllTimeTabStrict);
end;

Procedure BuildBitAddr(Reg : String; Bit : Byte; Name : String; Tab : TStrings);
Var I,X,B : Integer;
begin
  I:=AllTimeTabAll.IndexOf(ExtLowerCase(Reg));
  If I<0 then begin
    MessageDlg('Fehler beim Initialisieren der Bitadretabelle.',mtError,[mbOK],0); halt;
  end;
  I:=Integer(AllTimeTabAll.Objects[I]);

  B:=-1;
  For X:=0 to High(BitReg) do If I=BitReg[X] then begin B:=BitCode[X]; break; end;
  If B<0 then begin
    MessageDlg('Fehler beim Initialisieren der Bitadretabelle.',mtError,[mbOK],0); halt;
  end;

  Tab.AddObject(ExtLowerCase(Name),TObject(B+Bit));
end;

initialization
  AllTimeTabStrict:=TStringlist.Create;
  with AllTimeTabStrict do begin
    AddObject('acc',TObject(RegA));
    AddObject('b',TObject(RegB));
    AddObject('psw',TObject(RegPSW));
    AddObject('sp',TObject(RegSP));
    AddObject('dpl',TObject(RegDPL));
    AddObject('dph',TObject(RegDPL));
    AddObject('dpsel',TObject(RegDPSEL));
    AddObject('p0',TObject(RegP0));
    AddObject('p1',TObject(RegP1));
    AddObject('p2',TObject(RegP2));
    AddObject('p3',TObject(RegP3));
  end;
  AllTimeTabAll:=TStringlist.Create;
  with AllTimeTabAll do begin
    Assign(AllTimeTabStrict);
    AddObject('p1ana',TObject(RegP1Ana));
    AddObject('p3ana',TObject(RegP3Ana));
    AddObject('syscon',TObject(RegSYSCON));
    AddObject('p4',TObject(RegP4));
    AddObject('p5',TObject(RegP5));
    AddObject('ie',TObject(RegIEN0));
    AddObject('ien0',TObject(RegIEN0));
    AddObject('ien1',TObject(RegIEN1));
    AddObject('ip',TObject(RegIP0));
    AddObject('ip0',TObject(RegIP0));
    AddObject('ip1',TObject(RegIP1));
    AddObject('pcon',TObject(RegPCON));
    AddObject('pcon1',TObject(RegPCON1));
    AddObject('sbuf',TObject(RegSBUF));
    AddObject('scon',TObject(RegSCON));
    AddObject('tcon',TObject(RegTCON));
    AddObject('tmod',TObject(RegTMOD));
    AddObject('t2con',TObject(RegT2CON));
    AddObject('t2mod',TObject(RegT2MOD));
    AddObject('adcon',TObject(RegADCON0));
    AddObject('adcon0',TObject(RegADCON0));
    AddObject('adcon1',TObject(RegADCON1));
    AddObject('addath',TObject(RegADDATH));
    AddObject('addatl',TObject(RegADDATL));
    AddObject('itcon',TObject(RegITCON));
    AddObject('trcon',TObject(RegTRCON));
    AddObject('bcon',TObject(RegBCON));
    AddObject('ct2con',TObject(RegCT2CON));
    AddObject('ccir',TObject(RegCCIR));
    AddObject('tl0',TObject(RegTL0));
    AddObject('th0',TObject(RegTH0));
    AddObject('tl1',TObject(RegTL1));
    AddObject('th1',TObject(RegTH1));
    AddObject('ct1con',TObject(RegCT1CON));
    AddObject('ccpl',TObject(RegCCPL));
    AddObject('ccph',TObject(RegCCPH));
    AddObject('ccl0',TObject(RegCCL0));
    AddObject('cch0',TObject(RegCCH0));
    AddObject('ccl1',TObject(RegCCL1));
    AddObject('cch1',TObject(RegCCH1));
    AddObject('ccl2',TObject(RegCCL2));
    AddObject('cch2',TObject(RegCCH2));
    AddObject('ct1ofl',TObject(RegCT1OFL));
    AddObject('ct1ofh',TObject(RegCT1OFH));
    AddObject('cmsel0',TObject(RegCMSEL0));
    AddObject('cmsel1',TObject(RegCMSEL1));
  end;
  AllTimeBitTabStrict:=TStringlist.Create;
  BuildBitAddr('psw',0,'p',AllTimeBitTabStrict);
  BuildBitAddr('psw',1,'f1',AllTimeBitTabStrict);
  BuildBitAddr('psw',2,'ov',AllTimeBitTabStrict);
  BuildBitAddr('psw',3,'rs0',AllTimeBitTabStrict);
  BuildBitAddr('psw',4,'rs1',AllTimeBitTabStrict);
  BuildBitAddr('psw',5,'f0',AllTimeBitTabStrict);
  BuildBitAddr('psw',6,'ac',AllTimeBitTabStrict);
  BuildBitAddr('psw',7,'cy',AllTimeBitTabStrict);
  AllTimeBitTabAll:=TStringlist.Create;
  AllTimeBitTabAll.Assign(AllTimeBitTabStrict);
  BuildBitAddr('p3',2,'int0',AllTimeBitTabAll);
  BuildBitAddr('p3',3,'int1',AllTimeBitTabAll);
  BuildBitAddr('p3',4,'t0',AllTimeBitTabAll);
  BuildBitAddr('p3',5,'t1',AllTimeBitTabAll);
  BuildBitAddr('ien0',0,'ex0',AllTimeBitTabAll);
  BuildBitAddr('ien0',1,'et0',AllTimeBitTabAll);
  BuildBitAddr('ien0',2,'ex1',AllTimeBitTabAll);
  BuildBitAddr('ien0',3,'et1',AllTimeBitTabAll);
  BuildBitAddr('ien0',4,'es',AllTimeBitTabAll);
  BuildBitAddr('ien0',5,'et2',AllTimeBitTabAll);
  BuildBitAddr('ien0',7,'ea',AllTimeBitTabAll);
  BuildBitAddr('ip0',0,'px0',AllTimeBitTabAll);
  BuildBitAddr('ip0',1,'pt0',AllTimeBitTabAll);
  BuildBitAddr('ip0',2,'px1',AllTimeBitTabAll);
  BuildBitAddr('ip0',3,'pt1',AllTimeBitTabAll);
  BuildBitAddr('ip0',4,'ps',AllTimeBitTabAll);
  BuildBitAddr('ip0',5,'pt2',AllTimeBitTabAll);
  BuildBitAddr('adcon0',5,'iadc',AllTimeBitTabAll);
  BuildBitAddr('adcon0',4,'bsy',AllTimeBitTabAll);
  BuildBitAddr('adcon0',3,'adm',AllTimeBitTabAll);
  BuildBitAddr('tcon',1,'ie0',AllTimeBitTabAll);
  BuildBitAddr('tcon',3,'ie1',AllTimeBitTabAll);
  BuildBitAddr('tcon',4,'tr0',AllTimeBitTabAll);
  BuildBitAddr('tcon',5,'tf0',AllTimeBitTabAll);
  BuildBitAddr('tcon',6,'tr1',AllTimeBitTabAll);
  BuildBitAddr('tcon',7,'tf1',AllTimeBitTabAll);
  BuildBitAddr('t2con',6,'tr2',AllTimeBitTabAll);
  BuildBitAddr('t2con',7,'tf2',AllTimeBitTabAll);
  BuildBitAddr('scon',0,'ri',AllTimeBitTabAll);
  BuildBitAddr('scon',1,'ti',AllTimeBitTabAll);
finalization
  AllTimeTabAll.Free;
  AllTimeTabStrict.Free;
  AllTimeBitTabAll.Free;
  AllTimeBitTabStrict.Free;
end.
